@@ -1,19 +1,24 @@
 #!/usr/bin/env bash
 
-source $(dirname "$0")/tools.sh
+. $(dirname "$0")/tools.sh
 
-# Clone NSPR if needed.
-hg_clone https://hg.mozilla.org/projects/nspr ./nspr default
+set -e
+
+test -n "${VCS_PATH}"
+
+# builds write to the source dir (and its parent), so move the source trees to
+# our workspace from the (cached) checkout dir
+cp -a "${VCS_PATH}/nspr" "${VCS_PATH}/nss" .
 
 pushd nspr
 hg revert --all
-if [[ -f ../nss/nspr.patch && "$ALLOW_NSPR_PATCH" == "1" ]]; then
-  cat ../nss/nspr.patch | patch -p1
+if [ -f "../nss/nspr.patch" ] && [ "$ALLOW_NSPR_PATCH" = "1" ]; then
+  patch -p1 < ../nss/nspr.patch
 fi
 popd
 
 # Dependencies
-# For MacOS we have hardware in the CI which doesn't allow us o deploy VMs.
+# For MacOS we have hardware in the CI which doesn't allow us to deploy VMs.
 # The setup is hardcoded and can't be changed easily.
 # This part is a helper We install dependencies manually to help.
 if [ "$(uname)" = "Darwin" ]; then
@@ -26,10 +31,14 @@ fi
 nss/build.sh -g -v --enable-libpkix -Denable_draft_hpke=1 "$@"
 
 # Package.
-if [[ $(uname) = "Darwin" ]]; then
+if [ "$(uname)" = "Darwin" ]; then
   mkdir -p public
   tar cvfjh public/dist.tar.bz2 dist
 else
-  mkdir artifacts
+  if [ "$(uname)" = Linux ]; then
+    ln -s /builds/worker/artifacts artifacts
+  else
+    mkdir artifacts
+  fi
   tar cvfjh artifacts/dist.tar.bz2 dist
 fi
