@@ -71,52 +71,8 @@ Usage()
 /*
  * known attributes
  */
-static const CK_ATTRIBUTE_TYPE known_attributes[] = {
-    CKA_CLASS, CKA_TOKEN, CKA_PRIVATE, CKA_LABEL, CKA_APPLICATION,
-    CKA_VALUE, CKA_OBJECT_ID, CKA_CERTIFICATE_TYPE, CKA_ISSUER,
-    CKA_SERIAL_NUMBER, CKA_AC_ISSUER, CKA_OWNER, CKA_ATTR_TYPES, CKA_TRUSTED,
-    CKA_CERTIFICATE_CATEGORY, CKA_JAVA_MIDP_SECURITY_DOMAIN, CKA_URL,
-    CKA_HASH_OF_SUBJECT_PUBLIC_KEY, CKA_HASH_OF_ISSUER_PUBLIC_KEY,
-    CKA_CHECK_VALUE, CKA_KEY_TYPE, CKA_SUBJECT, CKA_ID, CKA_SENSITIVE,
-    CKA_ENCRYPT, CKA_DECRYPT, CKA_WRAP, CKA_UNWRAP, CKA_SIGN, CKA_SIGN_RECOVER,
-    CKA_VERIFY, CKA_VERIFY_RECOVER, CKA_DERIVE, CKA_START_DATE, CKA_END_DATE,
-    CKA_MODULUS, CKA_MODULUS_BITS, CKA_PUBLIC_EXPONENT, CKA_PRIVATE_EXPONENT,
-    CKA_PRIME_1, CKA_PRIME_2, CKA_EXPONENT_1, CKA_EXPONENT_2, CKA_COEFFICIENT,
-    CKA_PRIME, CKA_SUBPRIME, CKA_BASE, CKA_PRIME_BITS,
-    CKA_SUB_PRIME_BITS, CKA_VALUE_BITS, CKA_VALUE_LEN, CKA_EXTRACTABLE,
-    CKA_LOCAL, CKA_NEVER_EXTRACTABLE, CKA_ALWAYS_SENSITIVE,
-    CKA_KEY_GEN_MECHANISM, CKA_MODIFIABLE, CKA_EC_PARAMS,
-    CKA_EC_POINT, CKA_SECONDARY_AUTH, CKA_AUTH_PIN_FLAGS,
-    CKA_ALWAYS_AUTHENTICATE, CKA_WRAP_WITH_TRUSTED, CKA_WRAP_TEMPLATE,
-    CKA_UNWRAP_TEMPLATE, CKA_HW_FEATURE_TYPE, CKA_RESET_ON_INIT,
-    CKA_HAS_RESET, CKA_PIXEL_X, CKA_PIXEL_Y, CKA_RESOLUTION, CKA_CHAR_ROWS,
-    CKA_CHAR_COLUMNS, CKA_COLOR, CKA_BITS_PER_PIXEL, CKA_CHAR_SETS,
-    CKA_ENCODING_METHODS, CKA_MIME_TYPES, CKA_MECHANISM_TYPE,
-    CKA_REQUIRED_CMS_ATTRIBUTES, CKA_DEFAULT_CMS_ATTRIBUTES,
-    CKA_PKCS_TRUST_SERVER_AUTH, CKA_PKCS_TRUST_CLIENT_AUTH,
-    CKA_PKCS_TRUST_CODE_SIGNING, CKA_PKCS_TRUST_EMAIL_PROTECTION,
-    CKA_TRUST_IPSEC_IKE, CKA_PKCS_TRUST_TIME_STAMPING,
-    CKA_NAME_HASH_ALGORITHM, CKA_HASH_OF_CERTIFICATE,
-    CKA_SUPPORTED_CMS_ATTRIBUTES, CKA_NSS_URL, CKA_NSS_EMAIL,
-    CKA_NSS_SMIME_INFO, CKA_NSS_SMIME_TIMESTAMP,
-    CKA_NSS_PKCS8_SALT, CKA_NSS_PASSWORD_CHECK, CKA_NSS_EXPIRES,
-    CKA_NSS_KRL, CKA_NSS_PQG_COUNTER, CKA_NSS_PQG_SEED,
-    CKA_NSS_PQG_H, CKA_NSS_PQG_SEED_BITS, CKA_NSS_MODULE_SPEC,
-    CKA_NSS_TRUST_DIGITAL_SIGNATURE, CKA_NSS_TRUST_NON_REPUDIATION,
-    CKA_NSS_TRUST_KEY_ENCIPHERMENT, CKA_NSS_TRUST_DATA_ENCIPHERMENT,
-    CKA_NSS_TRUST_KEY_AGREEMENT, CKA_NSS_TRUST_KEY_CERT_SIGN,
-    CKA_NSS_TRUST_CRL_SIGN, CKA_NSS_TRUST_SERVER_AUTH,
-    CKA_NSS_TRUST_CLIENT_AUTH, CKA_NSS_TRUST_CODE_SIGNING,
-    CKA_NSS_TRUST_EMAIL_PROTECTION, CKA_NSS_TRUST_IPSEC_END_SYSTEM,
-    CKA_NSS_TRUST_IPSEC_TUNNEL, CKA_NSS_TRUST_IPSEC_USER,
-    CKA_NSS_TRUST_TIME_STAMPING, CKA_NSS_TRUST_STEP_UP_APPROVED,
-    CKA_NSS_CERT_SHA1_HASH, CKA_NSS_CERT_MD5_HASH,
-    CKA_NSS_DB, CKA_NSS_TRUST, CKA_NSS_OVERRIDE_EXTENSIONS,
-    CKA_PUBLIC_KEY_INFO
-};
-
-static unsigned int known_attributes_size = sizeof(known_attributes) /
-                                            sizeof(known_attributes[0]);
+extern const CK_ATTRIBUTE_TYPE sftkdb_known_attributes[];
+extern size_t sftkdb_known_attributes_size;
 
 PRBool
 isULONGAttribute(CK_ATTRIBUTE_TYPE type)
@@ -518,6 +474,7 @@ dumpLongAttribute(CK_ATTRIBUTE_TYPE type, CK_ULONG value)
             constType = ConstKeyType;
             break;
         case CKA_MECHANISM_TYPE:
+        case CKA_NAME_HASH_ALGORITHM:
             nameType = "CKM_NSS";
             constType = ConstMechanism;
             break;
@@ -651,8 +608,8 @@ dumpObject(CK_OBJECT_HANDLE id, SDB *db, SDB *keydb, PRBool isKey)
     char *alloc = NULL;
 
     printf("  Object 0x%08x:\n", (int)id);
-    for (i = 0; i < known_attributes_size; i++) {
-        CK_ATTRIBUTE_TYPE attribute = known_attributes[i];
+    for (i = 0; i < sftkdb_known_attributes_size; i++) {
+        CK_ATTRIBUTE_TYPE attribute = sftkdb_known_attributes[i];
         template.type = attribute;
         template.pValue = NULL;
         template.ulValueLen = 0;
