@@ -9,6 +9,8 @@
 #include "keyhi.h"
 #include "nss_scoped_ptrs.h"
 #include "pk11pub.h"
+#include "seccomon.h"
+#include "utilrename.h"
 
 #include "asn1/mutators.h"
 #include "base/database.h"
@@ -22,13 +24,29 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
   ScopedPK11SlotInfo slot(PK11_GetInternalSlot());
   assert(slot);
 
-  SECKEYPrivateKey *key = nullptr;
-  if (PK11_ImportDERPrivateKeyInfoAndReturnKey(slot.get(), &derPki, nullptr,
-                                               nullptr, false, false, KU_ALL,
-                                               &key, nullptr) == SECSuccess) {
-    SECKEY_DestroyPrivateKey(key);
+  SECKEYPrivateKey *privKey = nullptr;
+  if (PK11_ImportDERPrivateKeyInfoAndReturnKey(
+          slot.get(), &derPki, nullptr, nullptr, false, false, KU_ALL, &privKey,
+          nullptr) != SECSuccess) {
+    return 0;
   }
 
+  (void)SECKEY_PrivateKeyStrengthInBits(privKey);
+  (void)SECKEY_GetPrivateKeyType(privKey);
+  (void)PK11_SignatureLen(privKey);
+  (void)PK11_GetPrivateModulusLen(privKey);
+
+  ScopedSECKEYPublicKey pubKey(SECKEY_ConvertToPublicKey(privKey));
+  ScopedCERTCertificate cert(PK11_GetCertFromPrivateKey(privKey));
+
+  char *nickname = PK11_GetPrivateKeyNickname(privKey);
+  PORT_Free(nickname);
+
+  SECKEYPQGParams *params = PK11_GetPQGParamsFromPrivateKey(privKey);
+  PORT_FreeArena(params ? params->arena : nullptr, PR_FALSE);
+
+  SECKEY_DestroyPrivateKey(privKey);
+
   return 0;
 }
 
