@@ -13,7 +13,9 @@
 
 namespace nss_test {
 
-class Pkcs11KEMTest : public ::testing::Test {
+class Pkcs11KEMTest
+    : public ::testing::Test,
+      public ::testing::WithParamInterface<CK_NSS_KEM_PARAMETER_SET_TYPE> {
  protected:
   PK11SymKey *Encapsulate(const ScopedSECKEYPublicKey &pub,
                           CK_MECHANISM_TYPE target, PK11AttrFlags attrFlags,
@@ -65,22 +67,43 @@ class Pkcs11KEMTest : public ::testing::Test {
 
     SECITEM_FreeItem(&attrValue, PR_FALSE);
   }
+
+  CK_MECHANISM_TYPE keyGenMech() {
+    switch (GetParam()) {
+      case CKP_NSS_KYBER_768_ROUND3:
+        return CKM_NSS_KYBER_KEY_PAIR_GEN;
+      case CKP_NSS_ML_KEM_768:
+        return CKM_NSS_ML_KEM_KEY_PAIR_GEN;
+      default:
+        EXPECT_TRUE(false);
+        return 0;
+    }
+  }
+
+  CK_MECHANISM_TYPE encapsMech() {
+    switch (GetParam()) {
+      case CKP_NSS_KYBER_768_ROUND3:
+        return CKM_NSS_KYBER;
+      case CKP_NSS_ML_KEM_768:
+        return CKM_NSS_ML_KEM;
+      default:
+        EXPECT_TRUE(false);
+        return 0;
+    }
+  }
 };
 
-TEST_F(Pkcs11KEMTest, KemConsistencyTest) {
-  Pkcs11KeyPairGenerator generator(CKM_NSS_KYBER_KEY_PAIR_GEN);
+TEST_P(Pkcs11KEMTest, KemConsistencyTest) {
+  Pkcs11KeyPairGenerator generator(keyGenMech());
   ScopedSECKEYPrivateKey priv;
   ScopedSECKEYPublicKey pub;
   generator.GenerateKey(&priv, &pub, false);
 
-  ASSERT_EQ((unsigned int)KYBER768_PUBLIC_KEY_BYTES,
-            (unsigned int)pub->u.kyber.publicValue.len);
-
   // Copy the public key to simulate receiving the key as an octet string
   ScopedSECKEYPublicKey pubCopy(SECKEY_CopyPublicKey(pub.get()));
   ASSERT_NE(nullptr, pubCopy);
 
-  ScopedPK11SlotInfo slot(PK11_GetBestSlot(CKM_NSS_KYBER, nullptr));
+  ScopedPK11SlotInfo slot(PK11_GetBestSlot(encapsMech(), nullptr));
   ASSERT_NE(nullptr, slot);
 
   ASSERT_NE((unsigned int)CK_INVALID_HANDLE,
@@ -91,9 +114,6 @@ TEST_F(Pkcs11KEMTest, KemConsistencyTest) {
       pubCopy, CKM_SALSA20_POLY1305, PK11_ATTR_PRIVATE | PK11_ATTR_UNMODIFIABLE,
       CKF_ENCRYPT, &ciphertext));
 
-  ASSERT_EQ((unsigned int)KYBER768_CIPHERTEXT_BYTES,
-            (unsigned int)ciphertext->len);
-
   ASSERT_EQ(CKM_SALSA20_POLY1305, PK11_GetMechanism(sharedSecret.get()));
 
   CK_BBOOL ckTrue = CK_TRUE;
@@ -119,4 +139,13 @@ TEST_F(Pkcs11KEMTest, KemConsistencyTest) {
   EXPECT_EQ(0, SECITEM_CompareItem(item1, item2));
 }
 
+#ifndef NSS_DISABLE_KYBER
+INSTANTIATE_TEST_SUITE_P(Pkcs11KEMTest, Pkcs11KEMTest,
+                         ::testing::Values(CKP_NSS_KYBER_768_ROUND3,
+                                           CKP_NSS_ML_KEM_768));
+#else
+INSTANTIATE_TEST_SUITE_P(Pkcs11KEMTest, Pkcs11KEMTest,
+                         ::testing::Values(CKP_NSS_ML_KEM_768));
+#endif
+
 }  // namespace nss_test
