@@ -3,6 +3,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #include "cryptohi.h"
 #include "keyhi.h"
+#include "pkcs11t.h"
 #include "secoid.h"
 #include "secitem.h"
 #include "secder.h"
@@ -553,6 +554,11 @@ seckey_GetKeyType(SECOidTag tag)
         case SEC_OID_ED25519_PUBLIC_KEY:
             keyType = edKey;
             break;
+        case SEC_OID_ML_DSA_44_PUBLIC_KEY:
+        case SEC_OID_ML_DSA_65_PUBLIC_KEY:
+        case SEC_OID_ML_DSA_87_PUBLIC_KEY:
+            keyType = mldsaKey;
+            break;
         /* accommodate applications that hand us a signature type when they
          * should be handing us a cipher type */
         case SEC_OID_PKCS1_MD5_WITH_RSA_ENCRYPTION:
@@ -593,6 +599,82 @@ seckey_HasCurveOID(const SECKEYPublicKey *pubKey)
     return rv;
 }
 
+CK_ML_DSA_PARAMETER_SET_TYPE
+SECKEY_GetMLDSAPkcs11ParamSetByOidTag(SECOidTag tag)
+{
+    switch (tag) {
+        case SEC_OID_ML_DSA_44:
+            return CKP_ML_DSA_44;
+        case SEC_OID_ML_DSA_65:
+            return CKP_ML_DSA_65;
+        case SEC_OID_ML_DSA_87:
+            return CKP_ML_DSA_87;
+        default:
+            return CKP_INVALID_ID;
+    }
+}
+
+SECOidTag
+SECKEY_GetMLDSAOidTagByPkcs11ParamSet(CK_ML_DSA_PARAMETER_SET_TYPE paramSet)
+{
+    switch (paramSet) {
+        case CKP_ML_DSA_44:
+            return SEC_OID_ML_DSA_44;
+        case CKP_ML_DSA_65:
+            return SEC_OID_ML_DSA_65;
+        case CKP_ML_DSA_87:
+            return SEC_OID_ML_DSA_87;
+        default:
+            return SEC_OID_UNKNOWN;
+    }
+}
+
+unsigned int
+SECKEY_MLDSAOidParamsToLen(SECOidTag oid, SECKEYSizeType type)
+{
+    switch (type) {
+        case SECKEYPubKeyType:
+            switch (oid) {
+                case SEC_OID_ML_DSA_44:
+                    return ML_DSA_44_PUBLICKEY_LEN;
+                case SEC_OID_ML_DSA_65:
+                    return ML_DSA_65_PUBLICKEY_LEN;
+                case SEC_OID_ML_DSA_87:
+                    return ML_DSA_87_PUBLICKEY_LEN;
+                default:
+                    break;
+            }
+            break;
+        case SECKEYPrivKeyType:
+            switch (oid) {
+                case SEC_OID_ML_DSA_44:
+                    return ML_DSA_44_PRIVATEKEY_LEN;
+                case SEC_OID_ML_DSA_65:
+                    return ML_DSA_65_PRIVATEKEY_LEN;
+                case SEC_OID_ML_DSA_87:
+                    return ML_DSA_87_PRIVATEKEY_LEN;
+                default:
+                    break;
+            }
+            break;
+        case SECKEYSignatureType:
+            switch (oid) {
+                case SEC_OID_ML_DSA_44:
+                    return ML_DSA_44_SIGNATURE_LEN;
+                case SEC_OID_ML_DSA_65:
+                    return ML_DSA_65_SIGNATURE_LEN;
+                case SEC_OID_ML_DSA_87:
+                    return ML_DSA_87_SIGNATURE_LEN;
+                default:
+                    break;
+            }
+            break;
+        default:
+            break;
+    }
+    return 0;
+}
+
 static SECKEYPublicKey *
 seckey_ExtractPublicKey(const CERTSubjectPublicKeyInfo *spki)
 {
@@ -669,6 +751,24 @@ seckey_ExtractPublicKey(const CERTSubjectPublicKeyInfo *spki)
                 if (rv == SECSuccess)
                     return pubk;
                 break;
+            case SEC_OID_ML_DSA_44_PUBLIC_KEY:
+            case SEC_OID_ML_DSA_65_PUBLIC_KEY:
+            case SEC_OID_ML_DSA_87_PUBLIC_KEY:
+                /* A basic consistency check on inputs. */
+                if (spki->algorithm.parameters.len != 0 && newOs.len == 0) {
+                    PORT_SetError(SEC_ERROR_INPUT_LEN);
+                    break;
+                }
+
+                pubk->keyType = mldsaKey;
+                pubk->u.mldsa.paramSet = tag;
+
+                rv = SECITEM_CopyItem(arena, &pubk->u.mldsa.publicValue, &newOs);
+                if (rv != SECSuccess) {
+                    break;
+                }
+                return pubk;
+            case SEC_OID_X25519:
             case SEC_OID_ED25519_PUBLIC_KEY:
                 /* A basic consistency check on inputs. */
                 if (newOs.len == 0) {
@@ -682,18 +782,29 @@ seckey_ExtractPublicKey(const CERTSubjectPublicKeyInfo *spki)
                     break;
                 }
 
-                pubk->keyType = edKey;
+                /* edKey corresponds to Ed25519 mechanism.
+                 * ecMontKey corresponds to X25519 mechanism.
+                 * The other options are not supported. */
+                if (tag == SEC_OID_X25519) {
+                    pubk->keyType = ecMontKey;
+                } else {
+                    /* tag == SEC_OID_ED25519_PUBLIC_KEY */
+                    pubk->keyType = edKey;
+                }
+
                 pubk->u.ec.size = 0;
 
-                SECOidData *oidEd25519 = SECOID_FindOIDByTag(SEC_OID_ED25519_PUBLIC_KEY);
+                SECOidData *oid25519 = SECOID_FindOIDByTag(tag);
+                if (!oid25519) {
+                    break;
+                }
 
-                if (!SECITEM_AllocItem(arena, &pubk->u.ec.DEREncodedParams, oidEd25519->oid.len + 2)) {
-                    rv = SECFailure;
+                if (!SECITEM_AllocItem(arena, &pubk->u.ec.DEREncodedParams, oid25519->oid.len + 2)) {
                     break;
                 }
                 pubk->u.ec.DEREncodedParams.data[0] = SEC_ASN1_OBJECT_ID;
-                pubk->u.ec.DEREncodedParams.data[1] = oidEd25519->oid.len;
-                PORT_Memcpy(pubk->u.ec.DEREncodedParams.data + 2, oidEd25519->oid.data, oidEd25519->oid.len);
+                pubk->u.ec.DEREncodedParams.data[1] = oid25519->oid.len;
+                PORT_Memcpy(pubk->u.ec.DEREncodedParams.data + 2, oid25519->oid.data, oid25519->oid.len);
 
                 rv = SECITEM_CopyItem(arena, &pubk->u.ec.publicValue, &newOs);
                 if (rv != SECSuccess) {
@@ -873,6 +984,7 @@ SECKEY_ECParamsToKeySize(const SECItem *encodedParams)
         case SEC_OID_SECG_EC_SECT571R1:
             return 571;
 
+        case SEC_OID_X25519:
         case SEC_OID_CURVE25519:
         case SEC_OID_ED25519_PUBLIC_KEY:
             return 255;
@@ -1024,6 +1136,7 @@ SECKEY_ECParamsToBasePointOrderLen(const SECItem *encodedParams)
         case SEC_OID_SECG_EC_SECT571R1:
             return 570;
 
+        case SEC_OID_X25519:
         case SEC_OID_CURVE25519:
         case SEC_OID_ED25519_PUBLIC_KEY:
             return 255;
@@ -1098,8 +1211,14 @@ SECKEY_PublicKeyStrengthInBits(const SECKEYPublicKey *pubk)
             break;
         case ecKey:
         case edKey:
+        case ecMontKey:
             bitSize = SECKEY_ECParamsToKeySize(&pubk->u.ec.DEREncodedParams);
             break;
+        case mldsaKey:
+            bitSize = SECKEY_MLDSAOidParamsToLen(pubk->u.mldsa.paramSet,
+                                                 SECKEYPubKeyType) *
+                      8;
+            break;
         default:
             PORT_SetError(SEC_ERROR_INVALID_KEY);
             break;
@@ -1113,6 +1232,8 @@ SECKEY_PrivateKeyStrengthInBits(const SECKEYPrivateKey *privk)
     unsigned bitSize = 0;
     SECItem params = { siBuffer, NULL, 0 };
     SECStatus rv;
+    SECOidTag mlDsaOidTag;
+    CK_ML_DSA_PARAMETER_SET_TYPE mlDsaPkcs11ParamSet;
 
     if (!privk) {
         PORT_SetError(SEC_ERROR_INVALID_KEY);
@@ -1124,12 +1245,21 @@ SECKEY_PrivateKeyStrengthInBits(const SECKEYPrivateKey *privk)
         case rsaKey:
         case rsaPssKey:
         case rsaOaepKey:
-            /* some tokens don't export CKA_MODULUS on the private key,
-             * PK11_SignatureLen works around this if necessary */
-            bitSize = PK11_SignatureLen((SECKEYPrivateKey *)privk) * PR_BITS_PER_BYTE;
-            if (bitSize == -1) {
-                bitSize = 0;
+            rv = PK11_ReadAttribute(privk->pkcs11Slot, privk->pkcs11ID,
+                                    CKA_MODULUS, NULL, &params);
+            if ((rv != SECSuccess) || (params.data == NULL)) {
+                /* some tokens don't export CKA_MODULUS on the private key,
+                 * PK11_SignatureLen works around this if necessary. This
+                 * method is less percise because it returns bytes instead
+                 * of bits, so we only do it if we can't get the modulus */
+                bitSize = PK11_SignatureLen((SECKEYPrivateKey *)privk) * PR_BITS_PER_BYTE;
+                if (bitSize == -1) {
+                    return 0;
+                }
+                return bitSize;
             }
+            bitSize = SECKEY_BigIntegerBitLength(&params);
+            PORT_Free(params.data);
             return bitSize;
         case dsaKey:
         case fortezzaKey:
@@ -1153,6 +1283,21 @@ SECKEY_PrivateKeyStrengthInBits(const SECKEYPrivateKey *privk)
             bitSize = SECKEY_ECParamsToKeySize(&params);
             PORT_Free(params.data);
             return bitSize;
+        case mldsaKey:
+            mlDsaPkcs11ParamSet = PK11_ReadULongAttribute(privk->pkcs11Slot,
+                                                          privk->pkcs11ID,
+                                                          CKA_PARAMETER_SET);
+            if (mlDsaPkcs11ParamSet == CK_UNAVAILABLE_INFORMATION) {
+                break;
+            }
+            mlDsaOidTag = SECKEY_GetMLDSAOidTagByPkcs11ParamSet(mlDsaPkcs11ParamSet);
+            if (mlDsaOidTag == SEC_OID_UNKNOWN) {
+                break;
+            }
+            bitSize = SECKEY_MLDSAOidParamsToLen(mlDsaOidTag,
+                                                 SECKEYPrivKeyType) *
+                      8;
+            break;
         default:
             break;
     }
@@ -1180,10 +1325,15 @@ SECKEY_SignatureLen(const SECKEYPublicKey *pubk)
             return pubk->u.dsa.params.subPrime.len * 2;
         case ecKey:
         case edKey:
+        case ecMontKey:
             /* Get the base point order length in bits and adjust */
             size = SECKEY_ECParamsToBasePointOrderLen(
                 &pubk->u.ec.DEREncodedParams);
             return ((size + 7) / 8) * 2;
+        case mldsaKey:
+            size = SECKEY_MLDSAOidParamsToLen(pubk->u.mldsa.paramSet,
+                                              SECKEYSignatureType);
+            break;
         default:
             break;
     }
@@ -1307,6 +1457,7 @@ SECKEY_CopyPublicKey(const SECKEYPublicKey *pubk)
             break;
         case ecKey:
         case edKey:
+        case ecMontKey:
             copyk->u.ec.size = pubk->u.ec.size;
             rv = seckey_HasCurveOID(pubk);
             if (rv != SECSuccess) {
@@ -1321,6 +1472,11 @@ SECKEY_CopyPublicKey(const SECKEYPublicKey *pubk)
             rv = SECITEM_CopyItem(arena, &copyk->u.ec.publicValue,
                                   &pubk->u.ec.publicValue);
             break;
+        case mldsaKey:
+            copyk->u.mldsa.paramSet = pubk->u.mldsa.paramSet;
+            rv = SECITEM_CopyItem(arena, &copyk->u.mldsa.publicValue,
+                                  &pubk->u.mldsa.publicValue);
+            break;
         case nullKey:
             return copyk;
         case kyberKey:
@@ -1345,7 +1501,7 @@ SECKEY_CopyPublicKey(const SECKEYPublicKey *pubk)
  * size.
  */
 SECStatus
-seckey_EnforceKeySize(KeyType keyType, unsigned keyLength, SECErrorCodes error)
+SECKEY_EnforceKeySize(KeyType keyType, unsigned keyLength, SECErrorCodes error)
 {
     PRInt32 opt = -1;
     PRInt32 optVal;
@@ -1409,7 +1565,7 @@ seckey_FindPublicKeyHandle(SECKEYPrivateKey *privk, SECKEYPublicKey *pubk)
     pubk->pkcs11ID = PK11_DerivePubKeyFromPrivKey(privk);
     if (pubk->pkcs11ID == CK_INVALID_HANDLE) {
         /* end of the road. Token doesn't have matching public key, nor can
-          * token regenerate a new public key from and existing private key. */
+         * token regenerate a new public key from and existing private key. */
         return CK_INVALID_HANDLE;
     }
     pubk->pkcs11Slot = PK11_ReferenceSlot(privk->pkcs11Slot);
@@ -1425,6 +1581,8 @@ SECKEY_ConvertToPublicKey(SECKEYPrivateKey *privk)
     SECStatus rv;
     CK_OBJECT_HANDLE pubKeyHandle;
     SECItem decodedPoint;
+    CK_ML_DSA_PARAMETER_SET_TYPE mlDsaPkcs11ParamSet;
+    SECOidTag mlDsaOidTag;
 
     /*
      * First try to look up the cert.
@@ -1537,6 +1695,7 @@ SECKEY_ConvertToPublicKey(SECKEYPrivateKey *privk)
             pubk->u.ec.encoding = ECPoint_Undefined;
             return pubk;
         case edKey:
+        case ecMontKey:
             rv = PK11_ReadAttribute(privk->pkcs11Slot, privk->pkcs11ID,
                                     CKA_EC_PARAMS, arena, &pubk->u.ec.DEREncodedParams);
             if (rv != SECSuccess) {
@@ -1557,6 +1716,29 @@ SECKEY_ConvertToPublicKey(SECKEYPrivateKey *privk)
             }
             pubk->u.ec.encoding = ECPoint_Undefined;
             return pubk;
+        case mldsaKey:
+            pubKeyHandle = seckey_FindPublicKeyHandle(privk, pubk);
+            if (pubKeyHandle == CK_INVALID_HANDLE) {
+                break;
+            }
+            mlDsaPkcs11ParamSet = PK11_ReadULongAttribute(privk->pkcs11Slot,
+                                                          privk->pkcs11ID,
+                                                          CKA_PARAMETER_SET);
+            if (mlDsaPkcs11ParamSet == CK_UNAVAILABLE_INFORMATION) {
+                break;
+            }
+            mlDsaOidTag = SECKEY_GetMLDSAOidTagByPkcs11ParamSet(mlDsaPkcs11ParamSet);
+            if (mlDsaOidTag == SEC_OID_UNKNOWN) {
+                break;
+            }
+            pubk->u.mldsa.paramSet = mlDsaOidTag;
+            rv = PK11_ReadAttribute(privk->pkcs11Slot, pubKeyHandle,
+                                    CKA_VALUE, arena, &pubk->u.mldsa.publicValue);
+            if (rv != SECSuccess) {
+                break;
+            }
+            return pubk;
+
         default:
             break;
     }
@@ -1674,6 +1856,7 @@ seckey_CreateSubjectPublicKeyInfo_helper(SECKEYPublicKey *pubk)
                 }
                 break;
             case edKey:
+            case ecMontKey:
                 tag = SECKEY_GetECCOid(&pubk->u.ec.DEREncodedParams);
                 rv = SECOID_SetAlgorithmID(arena, &spki->algorithm,
                                            tag,
@@ -1685,6 +1868,30 @@ seckey_CreateSubjectPublicKeyInfo_helper(SECKEYPublicKey *pubk)
                 rv = SECITEM_CopyItem(arena, &spki->subjectPublicKey,
                                       &pubk->u.ec.publicValue);
 
+                if (rv == SECSuccess) {
+                    /*
+                     * The stored value is supposed to be a BIT_STRING,
+                     * so convert the length.
+                     */
+                    spki->subjectPublicKey.len <<= 3;
+                    /*
+                     * We got a good one; return it.
+                     */
+                    return spki;
+                }
+                break;
+            case mldsaKey:
+                tag = pubk->u.mldsa.paramSet;
+                rv = SECOID_SetAlgorithmID(arena, &spki->algorithm,
+                                           tag,
+                                           &params);
+                if (rv != SECSuccess) {
+                    break;
+                }
+
+                rv = SECITEM_CopyItem(arena, &spki->subjectPublicKey,
+                                      &pubk->u.mldsa.publicValue);
+
                 if (rv == SECSuccess) {
                     /*
                      * The stored value is supposed to be a BIT_STRING,
@@ -2310,10 +2517,18 @@ sec_GetHashMechanismByOidTag(SECOidTag tag)
     }
 }
 
-static CK_RSA_PKCS_MGF_TYPE
-sec_GetMgfTypeByOidTag(SECOidTag tag)
+CK_RSA_PKCS_MGF_TYPE
+SEC_GetMgfTypeByOidTag(SECOidTag tag)
 {
     switch (tag) {
+        case SEC_OID_SHA3_512:
+            return CKG_MGF1_SHA3_512;
+        case SEC_OID_SHA3_384:
+            return CKG_MGF1_SHA3_384;
+        case SEC_OID_SHA3_256:
+            return CKG_MGF1_SHA3_256;
+        case SEC_OID_SHA3_224:
+            return CKG_MGF1_SHA3_224;
         case SEC_OID_SHA512:
             return CKG_MGF1_SHA512;
         case SEC_OID_SHA384:
@@ -2415,7 +2630,8 @@ sec_DecodeRSAPSSParams(PLArenaPool *arena,
 SECStatus
 sec_DecodeRSAPSSParamsToMechanism(PLArenaPool *arena,
                                   const SECItem *params,
-                                  CK_RSA_PKCS_PSS_PARAMS *mech)
+                                  CK_RSA_PKCS_PSS_PARAMS *mech,
+                                  SECOidTag *hashAlgp)
 {
     SECOidTag hashAlg;
     SECOidTag maskHashAlg;
@@ -2427,13 +2643,14 @@ sec_DecodeRSAPSSParamsToMechanism(PLArenaPool *arena,
     if (rv != SECSuccess) {
         return SECFailure;
     }
+    *hashAlgp = hashAlg;
 
     mech->hashAlg = sec_GetHashMechanismByOidTag(hashAlg);
     if (mech->hashAlg == CKM_INVALID_MECHANISM) {
         return SECFailure;
     }
 
-    mech->mgf = sec_GetMgfTypeByOidTag(maskHashAlg);
+    mech->mgf = SEC_GetMgfTypeByOidTag(maskHashAlg);
     if (mech->mgf == 0) {
         return SECFailure;
     }
