@@ -193,7 +193,8 @@ struct SFTKObjectStr {
     SFTKSlot *slot;
     void *objectInfo;
     SFTKFree infoFree;
-    PRBool isFIPS;
+    CK_FLAGS validation_value;
+    SFTKAttribute validation_attribute;
 };
 
 struct SFTKTokenObjectStr {
@@ -292,6 +293,7 @@ struct SFTKSessionContextStr {
     SFTKVerify verify;
     unsigned int maxLen;
     SFTKObject *key;
+    SECItem *signature;
 };
 
 /*
@@ -505,6 +507,11 @@ struct SFTKItemTemplateStr {
 #define sftk_isFIPS(id) \
     (((id) == FIPS_SLOT_ID) || ((id) >= SFTK_MIN_FIPS_USER_SLOT_ID))
 
+/* validation flags. These are token specific, but also
+ * visible to the application via the validation object
+ * each validation should cover a unique bit. */
+#define SFTK_VALIDATION_FIPS_FLAG 0x00000001L
+
 /* the session hash multiplier (see bug 201081) */
 #define SHMULTIPLIER 1791398085
 
@@ -840,19 +847,19 @@ extern void sftk_EncodeInteger(PRUint64 integer, CK_ULONG num_bits, CK_BBOOL lit
 /* ike and xcbc helpers */
 extern CK_RV sftk_ike_prf(CK_SESSION_HANDLE hSession,
                           const SFTKAttribute *inKey,
-                          const CK_NSS_IKE_PRF_DERIVE_PARAMS *params, SFTKObject *outKey);
+                          const CK_IKE_PRF_DERIVE_PARAMS *params, SFTKObject *outKey);
 extern CK_RV sftk_ike1_prf(CK_SESSION_HANDLE hSession,
                            const SFTKAttribute *inKey,
-                           const CK_NSS_IKE1_PRF_DERIVE_PARAMS *params, SFTKObject *outKey,
+                           const CK_IKE1_PRF_DERIVE_PARAMS *params, SFTKObject *outKey,
                            unsigned int keySize);
 extern CK_RV sftk_ike1_appendix_b_prf(CK_SESSION_HANDLE hSession,
                                       const SFTKAttribute *inKey,
-                                      const CK_NSS_IKE1_APP_B_PRF_DERIVE_PARAMS *params,
+                                      const CK_IKE1_EXTENDED_DERIVE_PARAMS *params,
                                       SFTKObject *outKey,
                                       unsigned int keySize);
 extern CK_RV sftk_ike_prf_plus(CK_SESSION_HANDLE hSession,
                                const SFTKAttribute *inKey,
-                               const CK_NSS_IKE_PRF_PLUS_DERIVE_PARAMS *params, SFTKObject *outKey,
+                               const CK_IKE2_PRF_PLUS_DERIVE_PARAMS *params, SFTKObject *outKey,
                                unsigned int keySize);
 extern CK_RV sftk_aes_xcbc_new_keys(CK_SESSION_HANDLE hSession,
                                     CK_OBJECT_HANDLE hKey, CK_OBJECT_HANDLE_PTR phKey,
@@ -969,6 +976,10 @@ CK_FLAGS sftk_AttributeToFlags(CK_ATTRIBUTE_TYPE op);
  * FIPS security policy */
 PRBool sftk_operationIsFIPS(SFTKSlot *slot, CK_MECHANISM *mech,
                             CK_ATTRIBUTE_TYPE op, SFTKObject *source);
+/* manage the fips flag on objects */
+void sftk_setFIPS(SFTKObject *obj, PRBool isFIPS);
+PRBool sftk_hasFIPS(SFTKObject *obj);
+
 /* add validation objects to the slot */
 CK_RV sftk_CreateValidationObjects(SFTKSlot *slot);
 
